@@ -4,9 +4,11 @@ import sympy
 
 from .tools import execute_singular_command
 from .ring import Ring
+from .qring import QuotientRing
+from .ideal_algorithms import Ideal_Algorithms
 
 
-class Ideal(object):
+class Ideal(Ideal_Algorithms, object):
 
     def __init__(self, ring, generators):
         if not isinstance(ring, Ring) or not isinstance(generators, list):
@@ -43,9 +45,28 @@ class Ideal(object):
     def squoosh(self):
         self.generators = self.minbase
 
+    @functools.cached_property
+    def dim(self):
+        singular_commands = [f"ring r = {self.ring};",
+                             f"ideal gb = {','.join(self.groebner_basis)};",
+                             "print(dim(gb));",
+                             "$"]
+        output = execute_singular_command(singular_commands)
+        return int(output)
+
+    @functools.cached_property
+    def indepSet(self):
+        singular_commands = [f"ring r = {self.ring};",
+                             f"ideal gb = {','.join(self.groebner_basis)};",
+                             "print(indepSet(gb));",
+                             "$"]
+        output = execute_singular_command(singular_commands)
+        return tuple(map(int, output.split(",\n")))
+
     @functools.cached_property
     def indepSets(self):
         singular_commands = [f"ring r = {self.ring};",
+                             # f"ideal gb = {','.join(self.groebner_basis)};",   # this breaks singular variety construction, especially with mpcs
                              f"ideal i = {self};",
                              "ideal gb = groebner(i);",
                              "print(indepSet(gb, 1));",
@@ -84,9 +105,10 @@ class Ideal(object):
         singular_commands = ["LIB \"primdec.lib\";",
                              f"ring r = {self.ring};",
                              f"ideal i = {self};",
-                             "ideal gb = groebner(i);",
-                             "def pr = primdecGTZ(gb);",
-                             "print(pr);"]
+                             # f"ideal gb = {','.join(self.groebner_basis)};",
+                             "def pr = primdecGTZ(i);",   # options: GTZ / SY
+                             "print(pr);",
+                             "$"]
         output = execute_singular_command(singular_commands)
         output = [line.replace(",", "") for line in output.split("\n")]
 
@@ -137,8 +159,7 @@ class Ideal(object):
             other = str(other)
         assert isinstance(other, str)
         singular_commands = [f"ring r = {self.ring};",
-                             f"ideal i = {self};",
-                             "ideal gb = groebner(i);",
+                             f"ideal gb = {','.join(self.groebner_basis)};",
                              "poly f = " + other + ";",
                              "print(reduce(f, gb));"
                              "$"]
@@ -212,8 +233,32 @@ class Ideal(object):
         cls, ring = self.__class__, self.ring
         return cls(ring, output)
 
+    def reduce(self, other):
+        """Remainder of division, i.e. reduction."""
+        singular_commands = [f"ring r = {self.ring};",
+                             f"ideal gb = {','.join(self.groebner_basis)};",
+                             f"poly f = {other};",
+                             "poly g = reduce(f, gb);",
+                             "print(g);",
+                             "$"]
+        output = execute_singular_command(singular_commands)
+        return output
+
     def __str__(self):
         return ",".join(map(str, self.generators))
 
     def __repr__(self):
         return "Ideal over Ring\n    " + repr(self.ring) + "\ngenerated by\n    " + str(self)
+
+    def to_qring(self, other):
+        qring = QuotientRing(self.ring, other)
+        self.ring = qring
+
+    def to_full_ring(self):
+        if isinstance(self.ring, QuotientRing):
+            self.generators = (self.ring.ideal + self).generators
+            self.ring = self.ring.ideal.ring
+
+
+def reduce(poly, ideal):
+    return ideal.reduce(poly)
